﻿Shader "Simple/SimpleGrassWave" {
	Properties {
		_Color ("MainColor", Color) = (1,1,1,1)				// Unity側で色をセット
		_MainTex ("MainTexture", 2D) = "white" {}			// Unity側でテクスチャをセット
		_Cutoff ("AlphaCutoff", Range(0,1)) = 0.5			// Unity側でテクスチャの抜きレベルを調整
		_Cycle ("Cycle", Range(0.0,5.0) ) = 1.0				// Unity側で揺れのサイクルをセット
		_Amplitude ("Amplitude", Range(0,1.0) ) = 0.02		// Unity側で揺れの量をセット
	}

	SubShader { 
		// Queue 描画順(Background は 1000、Geometry は 2000、AlphaTest は 2450、Transparent は 3000、そして Overlay は 4000。数字が小さいほど先に描画される)
		// 描画タイプ。 Opaque=不透明　Transparent=半透明　TransparentCutout=抜き
		Tags { "Queue"="AlphaTest" "RenderType"="TransparentCutout" }

		// CGPROGRAM　〜　ENDCG  この中にシェーダーを書く
		CGPROGRAM

		// Surfaceと書くことででサーフェスシェーダーであることを宣言する。
		// surfの部分は色をいじる関数名を指定、vertの部分は頂点をいじる関数名を指定、Lambertの部分はライティングモードを指定。
		// alphatest:_Cutoffの部分で指定したアルファ値より大きいピクセルのみ描画するようにしている
		// ライトモードは、Lambert=拡散反射光　BlinnPhongは鏡面反射光
		#pragma surface surf Lambert alphatest:_Cutoff vertex:vert

		// Propertiesで宣言した変数を同名で宣言。UnityEditorから渡された値を扱えるようになる
		sampler2D _MainTex;
		float4 _Color;
		float _Cycle;
		float _Amplitude;

		// Unityからシェーダーに渡される構造体
		struct Input {
			float2 uv_MainTex;
			fixed4 color : COLOR;
		};

		// #pragmaで宣言したsurface関数 色に関する処理はここに書く
		// この関数の中を書き換えて質感を制御する
		void surf (Input IN, inout SurfaceOutput o) {
			half4 col = tex2D(_MainTex, IN.uv_MainTex) * _Color;
			o.Albedo = col;
			o.Alpha = col.a;
		}

		// #pragmaで宣言したvertex関数　頂点をいじる処理はここに書く
		void vert (inout appdata_full v) {
			// _TimeはShader内でアニメーションを行いたい時に利用　(x, y, z, w) が (t/20、t、t×2、t×3)となる。つまりyを使えばtそのままの値となる
			// 最後にv.vertex.yを乗算することで根元は揺らさず上部の揺れ幅を大きくしている
			v.vertex.x += sin((_Time.y * _Cycle)) * _Amplitude * v.vertex.y;	
			v.vertex.z += cos((_Time.y * _Cycle)) * _Amplitude * v.vertex.y;
		}   

		ENDCG
	}
}