﻿Shader "Simple/SimpleTextureShader" {
	Properties {
		_Color ("Color", Color) = (1,1,1,1)        // Unity側で色をセット
		_MainTex ("Texture", 2D) = "white" {}    // Unity側でテクスチャをセット
	}
	SubShader {
		// 描画タイプ。 Opaque=不透明　Transparent=半透明　TransparentCutout=抜き
		Tags { "RenderType"="Transparent" "Queue" = "Transparent" }    

		// CGPROGRAM　〜　ENDCG  この中にシェーダーを書く
		CGPROGRAM    

		// Surfaceと書くことででサーフェスシェーダーであることを宣言する。
		// surfの部分は関数名、Lambertの部分はライティングモードを指定。
		// ライトモードは、Lambert=拡散反射光　BlinnPhongは鏡面反射光
		#pragma surface surf Lambert alpha

		// Unityからシェーダーに渡される構造体
		struct Input {
			float4 color : COLOR;    // 色
			float2 uv_MainTex;        // Propertiesで宣言した変数の前に、uv をつけることで、マテリアル情報が適用された座標を受け取れる
		};

		// Propertiesで宣言した変数を同名で宣言。UnityEditorから渡された値を扱えるようになる
		fixed4 _Color;
		sampler2D _MainTex;


		// #pragmaで宣言したsurface関数
		// この関数の中を書き換えて質感を制御する
		void surf (Input IN, inout SurfaceOutput o) {
			// tex2D関数：UV座標(IN.uv_MainTex)からテクスチャ(_MainTex)上のピクセルの色を計算して返す 
			o.Albedo = tex2D (_MainTex, IN.uv_MainTex).rgb * _Color;
			o.Alpha = _Color.a * tex2D (_MainTex, IN.uv_MainTex).a;
		}
		ENDCG
	}
	FallBack "Diffuse"
}