﻿Shader "Simple/SimpleWaterWave.shader" {
	Properties {
		_Color ("MainColor", Color) = (1,1,1,1)				// Unity側で色をセット
		_Cycle ("Cycle", Range(0.0,5.0) ) = 1.0				// Unity側で揺れのサイクルをセット
		_Amplitude ("Amplitude", Range(0,1.0) ) = 0.02		// Unity側で揺れの量をセット
	}

	SubShader { 
		// Queue 描画順(Background は 1000、Geometry は 2000、AlphaTest は 2450、Transparent は 3000、そして Overlay は 4000。数字が小さいほど先に描画される)
		// 描画タイプ。 Opaque=不透明　Transparent=半透明　TransparentCutout=抜き
		Tags { "Queue"="Transparent" "RenderType"="Transparent" }

		// CGPROGRAM　〜　ENDCG  この中にシェーダーを書く
		CGPROGRAM

		// Surfaceと書くことででサーフェスシェーダーであることを宣言する。
		// surfの部分は色をいじる関数名を指定、vertの部分は頂点をいじる関数名を指定、Lambertの部分はライティングモードを指定。
		// ライトモードは、Lambert=拡散反射光　BlinnPhongは鏡面反射光
		#pragma surface surf Lambert vertex:vert alpha

		// Propertiesで宣言した変数を同名で宣言。UnityEditorから渡された値を扱えるようになる
		float4 _Color;
		float _Cycle;
		float _Amplitude;

		// Unityからシェーダーに渡される構造体
		struct Input {
			fixed4 color : COLOR;
		};

		// #pragmaで宣言したsurface関数 色に関する処理はここに書く
		// この関数の中を書き換えて質感を制御する
		void surf (Input IN, inout SurfaceOutput o) {
			o.Albedo = _Color;
			o.Alpha = _Color.a;
		}

		// #pragmaで宣言したvertex関数　頂点をいじる処理はここに書く
		void vert (inout appdata_full v) {
			// _TimeはShader内でアニメーションを行いたい時に利用　(x, y, z, w) が (t/20、t、t×2、t×3)となる。つまりyを使えばtそのままの値となる
			// sin関数内でv.vertex.xとv.vertex.zを乗算することで頂点の位置によってい揺れるタイミングを変えている
			v.vertex.y += sin((_Time.y * _Cycle * v.vertex.x * v.vertex.z)) * _Amplitude;	
		}   

		ENDCG
	}
}